import 'package:flutter/material.dart';
import 'package:taboola_sdk/taboola.dart';
import 'package:taboola_sdk_example/DefaultProperties.dart'
    as DefaultProperties;
import 'package:taboola_sdk_example/MidArticleWithoutVideo.dart'
    as MidArticlePublisherProperties;
import 'package:taboola_sdk/classic/taboola_classic.dart';
import 'package:taboola_sdk/classic/taboola_classic_listener.dart';

class TaboolaDedupScreen extends StatefulWidget {
  _TaboolaDedupScreenState createState() => _TaboolaDedupScreenState();
}

class _TaboolaDedupScreenState extends State<TaboolaDedupScreen> {
  ScaffoldMessengerState
      _scaffoldMessenger; //A reference to ScaffoldMessenger to allow sending Snackbar messages
  Widget _firstWidget;
  bool _firstWidgetLoaded =
      false; // Indicate that first widget on the screen finish loading and we can call the second one
  ScrollController _scrollController;
  TaboolaClassicListener _taboolaClassicListener;
  TaboolaClassicBuilder _taboolaClassicBuilder;

  @override
  Widget build(BuildContext context) {
    Taboola.init(PublisherInfo(DefaultProperties.publisher));
    _taboolaClassicBuilder = Taboola.getTaboolaClassicBuilder(
        DefaultProperties.pageUrl, DefaultProperties.pageType);

    _taboolaClassicListener = TaboolaClassicListener(taboolaDidResize,
        taboolaDidShow, taboolaDidFailToLoad, taboolaDidClickOnItem);

    //Define ScaffoldMessanger
    _scaffoldMessenger = ScaffoldMessenger.of(context);
    _scrollController = ScrollController();
    if (!_firstWidgetLoaded) {
      return CustomScrollView(
        controller: _scrollController,
        slivers: [
          SliverToBoxAdapter(
              child: new Container(
                  child: buildFirstWidget(
                      _taboolaClassicBuilder, _taboolaClassicListener))),
        ],
      );
    } else {
      return CustomScrollView(
        controller: _scrollController,
        slivers: [
          SliverToBoxAdapter(
              child: new Container(
                  child: buildFirstWidget(
                      _taboolaClassicBuilder, _taboolaClassicListener))),
          SliverToBoxAdapter(
              child: new Container(
                  child: buildSecondWidget(
                      _taboolaClassicBuilder, _taboolaClassicListener))),
        ],
      );
    }
  }

  /**
   * Function build first widget if already build will return the same on.
   * this is for preventing the first widget to recreate after it finished fetching data and we update layout to add new widgets
   */
  Widget buildFirstWidget(TaboolaClassicBuilder taboolaClassicBuilder,
      TaboolaClassicListener taboolaClassicListener) {
    if (_firstWidget == null) {
      _firstWidget = taboolaClassicBuilder.build(
          MidArticlePublisherProperties.placement_widget,
          MidArticlePublisherProperties.mode,
          true,
          taboolaClassicListener,
          viewId: 123456789);
    }
    return _firstWidget;
  }

  Widget buildSecondWidget(TaboolaClassicBuilder taboolaClassicBuilder,
      TaboolaClassicListener taboolaClassicListener) {
    return taboolaClassicBuilder.build(DefaultProperties.placement,
        DefaultProperties.mode, true, taboolaClassicListener,
        viewId: 123456789);
  }

  //Taboola Standard listeners
  void taboolaDidShow(String placement) {
    print("taboolaDidShow");
    if (placement == MidArticlePublisherProperties.placement_widget) {
      setState(() {
        _firstWidgetLoaded = true;
      });
    }
  }

  void taboolaDidResize(String placement, double height) {
    print("publisher did get height $height");
  }

  void taboolaDidFailToLoad(String placement, String error) {
    print("publisher placement:$placement did fail with an error:$error");
  }

  bool taboolaDidClickOnItem(
      String placement, String itemId, String clickUrl, bool organic) {
    print(
        "publisher did click on item: $itemId with clickUrl: $clickUrl in placement: $placement of organic: $organic");
    if (organic) {
      _showToast("Publisher opted to open click but didn't actually open it.");
    } else {
      _showToast(
          "Publisher opted to open clicks but the item is Sponsored, SDK retains control.");
    }
    return false;
  }

  void _showToast(String text) {
    _scaffoldMessenger.showSnackBar(
      SnackBar(
        content: Text(text),
        action: SnackBarAction(
            label: 'OK', onPressed: _scaffoldMessenger.hideCurrentSnackBar),
      ),
    );
  }
}
