import 'dart:io' show Platform;
import 'package:flutter/cupertino.dart';
import 'package:flutter/gestures.dart';
import 'dart:core';

import 'package:taboola_sdk/classic/taboola_webview.dart';
import 'package:taboola_sdk/utils/logger.dart';

class WebViewGestureRecognizer extends OneSequenceGestureRecognizer {
  TaboolaWebView _taboolaWebView;

  WebViewGestureRecognizer(TaboolaWebView taboolaWebView) {
    _taboolaWebView = taboolaWebView;
  }

  //Follow WebView global y scrolling value
  double yGlobal = 0;
  Offset dragDistance = Offset.zero;

  /// This is the only event that we managed to switch scrolling to web content
  @override
  void addPointer(PointerEvent event) {
    startTrackingPointer(
        event.pointer); //Allow handleEvent() to handle the logic.
  }

  @override
  String get debugDescription => 'customPan';

  @override
  void didStopTrackingLastPointer(int pointer) {}

  bool canWVContentScrollUp = false;
  double mOldPointY = 0, mOldPointX = 0, mDeltaX, mDeltaY;
  @override
  void handleEvent(PointerEvent event) async {
    dragDistance = dragDistance + event.delta;
    if (isWebViewAtScreenTop()) {
      bool horizontalScrolling = false;
      switch (event.runtimeType) {
        case PointerDownEvent:
          mOldPointY = event.position.dy;
          mOldPointX = event.position.dx;
          break;
        case PointerMoveEvent:
          mDeltaX = mOldPointX - event.position.dx;
          mDeltaY = mOldPointY - event.position.dy;
          horizontalScrolling = isHorizontalScrolling(mDeltaX, mDeltaY);
          break;
      }

      if (event.runtimeType != PointerMoveEvent) {
        //Don't switch scroll control when not scrolling (pointer moving). This was implemented to prevent a need for an extra down scroll gesture when moving from native into web-content scroll.
        return;
      }

      yGlobal = dragDistance.dy;
      int scrollDirection = getScrollingDirection(mDeltaY);
      if (scrollDirection == _SCROLL_DIRECTION_DOWN) {
        // This works since we run this method inside a condition that the WebView is already at screen top
        canWVContentScrollUp = true;
      }

      if (horizontalScrolling) {
        resolve(GestureDisposition.rejected); // Native internal will win
      } else if (scrollDirection == _SCROLL_DIRECTION_UP &&
          !canWVContentScrollUp) {
        //DeltaY<0 is user trying to scroll up + WebView content cannot scroll up: return touch to Native.
        resolve(GestureDisposition.rejected); // Native internal will win
      } else {
        resolve(GestureDisposition.accepted); // WebView internal will win
//        canWVContentScrollUp = true;
      }
    }
  }

  static const int _SCROLL_DIRECTION_UP = 1;
  static const int _SCROLL_DIRECTION_NONE = 1;
  static const int _SCROLL_DIRECTION_DOWN = -1;
  int getScrollingDirection(double touchMovementDeltaY) {
    if (touchMovementDeltaY < 0) {
      return _SCROLL_DIRECTION_UP;
    } else if (touchMovementDeltaY > 0) {
      return _SCROLL_DIRECTION_DOWN;
    } else {
      return _SCROLL_DIRECTION_NONE;
    }
  }

  ///This event is eventually called from MobileLoader, when the WebView's content reached top
  void onWebViewScrolledToContentTop() {
    Logger.log('onWebViewScrolledToContentTop');
    if (Platform.isAndroid) {
      canWVContentScrollUp = false;
//      // Set y position to 0 because the webView scrolled to top
      yGlobal = 0;
      dragDistance = Offset.zero;
    }
  }

  bool isWebViewAtScreenTop() {
    //TODO: try to compare to parent scrollView y offset (this will help with stuff like ActionBar, etc..)
    return _taboolaWebView.getPositionOnScreen().dy <= 1;
  }

  bool isHorizontalScrolling(double mDeltaX, double mDeltaY) {
    // kTouchSlop: "The distance a touch has to travel for the framework to be confident that the gesture is a scroll gesture". Source: constants.dart.
    if (mDeltaX > kTouchSlop && mDeltaX > mDeltaY) {
      return true;
    }
    return false;
  }
}
